/*****************************************************************************
Copyright (c) 2011 SmartVault, Inc.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ******************************************************************************/
package com.smartvault.rest;

import com.smartvault.oauth.SelfTokenEncoder;

import com.smartvault.net.WebRequestInfo;
import com.smartvault.net.ClientNetContext;

import com.smartvault.proto.NonceProto.NonceResponse;
import com.smartvault.proto.NonceProto.NonceRequest;
import com.smartvault.proto.SelfTokenProto.SelfTokenRequest;
import com.smartvault.proto.ClientTokenProto.ClientTokenResponse;

/**
 * This class performs the OAuth 2.0 autonomous authentication. This allows
 * your client program to interact with the REST API. This step should only be
 * performed on a trusted server.
 * 
 * Typical use is:
 * <code>
 * Authentication auth = new Authentication(cfg);
 * ClientProtocol client = auth.createClientProtocol();
 * </code>
 * 
 * @see ServerCfg
 */
public class Authentication {
	private ServerCfg cfg;

	/**
	 * Constructor
	 * 
	 * @param cfg the configuration of your client program
	 */
	public Authentication(ServerCfg cfg) {
		this.cfg = cfg;
	}

	/**
	 * performs the OAuth 2.0 autonomous authentication. The will produce
	 * a ClientCfg object that can be passed to a ClientProtocol object.
	 * This allows your client program to interact with the REST API.
	 * 
	 * @return
	 * @throws Exception
	 * 
	 * @see ClientCfg
	 * @see ClientProtocol
	 */
	public ClientCfg getClientAccessToken() throws Exception {
		WebRequestInfo info = new WebRequestInfo(this.cfg.getRootUri(), null, null);

		ClientNetContext ctx = ClientNetContext.open(info, Constants.NONCE_PATH,
				ClientNetContext.POST);

		NonceRequest.Builder bld = NonceRequest.newBuilder();
		bld.setClientId(this.cfg.getClientId());

		ctx.writePayload(bld.build());

		NonceResponse response = ctx.readPayloadNonceResponse();


		ctx = ClientNetContext.open(info, Constants.CLIENT_TOKEN_PATH,
				ClientNetContext.POST);

		SelfTokenEncoder ste = new SelfTokenEncoder(this.cfg.getClientId(),
				response.getMessage().getCode());

		SelfTokenRequest.Builder bld2 = SelfTokenRequest.newBuilder();
		bld2.setToken(ste.generate(this.cfg.getPrivateKey()));

		ctx.writePayload(bld2.build());

		ClientTokenResponse response2 = ctx.readPayloadClientTokenResponse();


		return new ClientCfg(this.cfg.getClientId(), response2.getMessage()
				.getToken(), this.cfg);
	}

	/**
	 * performs the OAuth 2.0 autonomous authentication. Creates a ClientProtocol
	 * object. This allows your client program to interact with the REST API.
	 * 
	 * @return
	 * @throws Exception
	 * 
	 * @see ClientCfg
	 * @see ClientProtocol	 
	 */
	public ClientProtocol createClientProtocol() throws Exception {
		return new ClientProtocol(getClientAccessToken());
	}
}